/* eslint-env node */

const regexCache = {}
const getRegex = pattern => {
  if (regexCache[pattern]) {
    return regexCache[pattern]
  }
  if (pattern.startsWith('/') && pattern.endsWith('/')) {
    // 1. /xxxx/ 格式的认为直接就是正则，需包含 command + count 捕获组
    // e.g. /^([(（](?<count>\d+)[)）])?(?<command>.+?)([(（](?<count2>\d+)[)）])?$/
    return regexCache[pattern] = new RegExp(pattern.substring(1, pattern.length - 1))
  } else {
    // 2. 普通格式的，拼装正则
    return regexCache[pattern] = new RegExp(`^(?<command>.+?)${pattern}(?<count>\\d+)$`)
  }
}

const handled = Symbol()

module.exports = ({ dispatchUserCommand, getPreference, _ }) => {
  const cloneUserCommand = cmd => {
    // 兼容 v2 版本，satori 内部对象无法被 cloneDeep，我们只关注 IUserCommand 这个 interface 需要的字段
    const result = { command: cmd.command, context: _.cloneDeep(cmd.context) }
    for (const key of Reflect.ownKeys(cmd)) {
      if (['session', 'command', 'substitute', 'bot'].includes(key)) continue
      result[key] = cmd[key]
    }
    return result
  }

  return {
    id: 'io.paotuan.plugin.globalflags',
    name: '全局指令选项',
    description: '在任意指令结尾加上 -x2 可进行连续掷骰',
    version: 2,
    preference: [
      {
        key: 'pattern',
        label: '触发格式',
        defaultValue: '\\-x'
      },
    ],
    hook: {
      onReceiveCommand: [
        {
          id: 'x',
          name: '全局连续掷骰',
          description: '在任意指令结尾加上 -x2 可进行连续掷骰',
          handler: result => {
            // 只处理一次
            if (result[handled]) {
              return false
            }
            result[handled] = true
            const pattern = getPreference(result.context).pattern
            const regex = getRegex(pattern)
            const matchResult = result.command.match(regex)
            if (matchResult && matchResult.groups) {
              let newCommand = undefined
              let count = undefined
              Object.keys(matchResult.groups).forEach(key => {
                // 所有 command* 代表新的 command
                // 所有 count* 代表新的 count
                if (key.startsWith('command')) {
                  const str = matchResult.groups[key]
                  if (typeof str !== 'undefined') {
                    newCommand = str.trim()
                  }
                } else if (key.startsWith('count')) {
                  const str = matchResult.groups[key]
                  if (typeof str !== 'undefined') {
                    count = parseInt(str || '1')
                  }
                }
              })
              if (newCommand && count) {
                console.log('[全局连续掷骰]指令:', newCommand, '次数:', count)
                // 修改本次指令
                result.command = newCommand
                // 派发余下的次数
                const newResultToDispatch = cloneUserCommand(result)
                setTimeout( async () => {
                  for (let i = 0; i < count - 1; i++) {
                    // 防止触发频控，每发 5 条停止一下
                    if (i > 0 && i % 5 === 0) {
                      await new Promise(resolve => setTimeout(resolve, 1000))
                    }
                    dispatchUserCommand(cloneUserCommand(newResultToDispatch))
                  }
                }, 10)
                return true
              }
            }
            return false
          }
        }
      ]
    }
  }
}
